@@ -1,71 +1,120 @@
-import builtins
 import sys
 from collections import deque
+from collections.abc import Mapping as AbcMapping
+from collections.abc import MutableMapping as AbcMutableMapping
 from collections.abc import MutableSet as AbcMutableSet
 from collections.abc import Set as AbcSet
-from dataclasses import MISSING
+from dataclasses import MISSING, Field, is_dataclass
 from dataclasses import fields as dataclass_fields
-from dataclasses import is_dataclass
+from functools import partial
+from inspect import signature as _signature
 from typing import AbstractSet as TypingAbstractSet
-from typing import Any, Deque, Dict, FrozenSet, List
+from typing import (
+    Any,
+    Deque,
+    Dict,
+    Final,
+    FrozenSet,
+    List,
+    Literal,
+    NewType,
+    Optional,
+    Protocol,
+    Tuple,
+    Type,
+    Union,
+    get_args,
+    get_origin,
+    get_type_hints,
+)
 from typing import Mapping as TypingMapping
 from typing import MutableMapping as TypingMutableMapping
 from typing import MutableSequence as TypingMutableSequence
 from typing import MutableSet as TypingMutableSet
-from typing import NewType, Optional
 from typing import Sequence as TypingSequence
 from typing import Set as TypingSet
-from typing import Tuple, get_type_hints
 
-from attr import NOTHING, Attribute, Factory
-from attr import fields as attrs_fields
-from attr import resolve_types
-
-try:
-    from typing_extensions import TypedDict as ExtensionsTypedDict
-except ImportError:
-    ExtensionsTypedDict = None
-
-try:
-    from typing_extensions import _TypedDictMeta as ExtensionsTypedDictMeta
-except ImportError:
-    ExtensionsTypedDictMeta = None
+from attrs import NOTHING, Attribute, Factory, resolve_types
+from attrs import fields as attrs_fields
+from attrs import fields_dict as attrs_fields_dict
 
 __all__ = [
+    "ANIES",
+    "adapted_fields",
+    "fields_dict",
+    "ExceptionGroup",
     "ExtensionsTypedDict",
-    "is_py37",
-    "is_py38",
-    "is_py39_plus",
-    "is_py310_plus",
-    "is_py311_plus",
+    "get_type_alias_base",
+    "has",
+    "is_type_alias",
     "is_typeddict",
+    "TypeAlias",
     "TypedDict",
 ]
 
-version_info = sys.version_info[0:3]
-is_py37 = version_info[:2] == (3, 7)
-is_py38 = version_info[:2] == (3, 8)
-is_py39_plus = version_info[:2] >= (3, 9)
-is_py310_plus = version_info[:2] >= (3, 10)
-is_py311_plus = version_info[:2] >= (3, 11)
-
-if is_py37:
-
-    def get_args(cl):
-        return cl.__args__
-
-    def get_origin(cl):
-        return getattr(cl, "__origin__", None)
-
-    from typing_extensions import Final, Protocol
+try:
+    from typing_extensions import TypedDict as ExtensionsTypedDict
+except ImportError:  # pragma: no cover
+    ExtensionsTypedDict = None
 
+if sys.version_info >= (3, 11):
+    from builtins import ExceptionGroup
 else:
-    from typing import Final, Protocol, get_args, get_origin
-
-if "ExceptionGroup" not in dir(builtins):
     from exceptiongroup import ExceptionGroup
-else:
-    ExceptionGroup = ExceptionGroup
+
+try:
+    from typing_extensions import is_typeddict as _is_typeddict
+except ImportError:  # pragma: no cover
+    assert sys.version_info >= (3, 10)
+    from typing import is_typeddict as _is_typeddict
+
+try:
+    from typing_extensions import TypeAlias
+except ImportError:  # pragma: no cover
+    assert sys.version_info >= (3, 11)
+    from typing import TypeAlias
+
+LITERALS = {Literal}
+try:
+    from typing_extensions import Literal as teLiteral
+
+    LITERALS.add(teLiteral)
+except ImportError:  # pragma: no cover
+    pass
+
+# On some Python versions, `typing_extensions.Any` is different than
+# `typing.Any`.
+try:
+    from typing_extensions import Any as teAny
+
+    ANIES = frozenset([Any, teAny])
+except ImportError:  # pragma: no cover
+    ANIES = frozenset([Any])
+
+NoneType = type(None)
+
+
+def is_optional(typ: Type) -> bool:
+    return is_union_type(typ) and NoneType in typ.__args__ and len(typ.__args__) == 2
+
+
+def is_typeddict(cls):
+    """Thin wrapper around typing(_extensions).is_typeddict"""
+    return _is_typeddict(getattr(cls, "__origin__", cls))
+
+
+def is_type_alias(type: Any) -> bool:
+    """Is this a PEP 695 type alias?"""
+    return False
+
+
+def get_type_alias_base(type: Any) -> Any:
+    """
+    What is this a type alias of?
+
+    Works only on 3.12+.
+    """
+    return type.__value__
 
 
 def has(cls):
@@ -81,13 +130,17 @@ def fields(type):
     try:
         return type.__attrs_attrs__
     except AttributeError:
-        try:
-            return dataclass_fields(type)
-        except AttributeError:
-            raise Exception("Not an attrs or dataclass class.")
+        return dataclass_fields(type)
 
 
-def _adapted_fields(cl) -> List[Attribute]:
+def fields_dict(type) -> Dict[str, Union[Attribute, Field]]:
+    """Return the fields_dict for attrs and dataclasses."""
+    if is_dataclass(type):
+        return {f.name: f for f in dataclass_fields(type)}
+    return attrs_fields_dict(type)
+
+
+def adapted_fields(cl) -> List[Attribute]:
     """Return the attrs format of `fields()` for attrs and dataclasses."""
     if is_dataclass(cl):
         attrs = dataclass_fields(cl)
@@ -100,12 +153,14 @@ def _adapted_fields(cl) -> List[Attribute]:
         return [
             Attribute(
                 attr.name,
-                attr.default
-                if attr.default is not MISSING
-                else (
-                    Factory(attr.default_factory)
-                    if attr.default_factory is not MISSING
-                    else NOTHING
+                (
+                    attr.default
+                    if attr.default is not MISSING
+                    else (
+                        Factory(attr.default_factory)
+                        if attr.default_factory is not MISSING
+                        else NOTHING
+                    )
                 ),
                 None,
                 True,
@@ -114,16 +169,17 @@ def _adapted_fields(cl) -> List[Attribute]:
                 attr.init,
                 True,
                 type=type_hints.get(attr.name, attr.type),
+                alias=attr.name,
+                kw_only=getattr(attr, "kw_only", False),
             )
             for attr in attrs
         ]
-    else:
+    attribs = attrs_fields(cl)
+    if any(isinstance(a.type, str) for a in attribs):
+        # PEP 563 annotations - need to be resolved.
+        resolve_types(cl)
         attribs = attrs_fields(cl)
-        if any(isinstance(a.type, str) for a in attribs):
-            # PEP 563 annotations - need to be resolved.
-            resolve_types(cl)
-            attribs = attrs_fields(cl)
-        return attribs
+    return attribs
 
 
 def is_subclass(obj: type, bases) -> bool:
@@ -140,7 +196,7 @@ def is_hetero_tuple(type: Any) -> bool:
 
 
 def is_protocol(type: Any) -> bool:
-    return issubclass(type, Protocol) and getattr(type, "_is_protocol", False)
+    return is_subclass(type, Protocol) and getattr(type, "_is_protocol", False)
 
 
 def is_bare_final(type) -> bool:
@@ -151,181 +207,51 @@ def get_final_base(type) -> Optional[type]:
     """Return the base of the Final annotation, if it is Final."""
     if type is Final:
         return Any
-    elif type.__class__ is _GenericAlias and type.__origin__ is Final:
+    if type.__class__ is _GenericAlias and type.__origin__ is Final:
         return type.__args__[0]
-    else:
-        return None
+    return None
 
 
 OriginAbstractSet = AbcSet
 OriginMutableSet = AbcMutableSet
 
-if is_py37 or is_py38:
-    Set = TypingSet
-    AbstractSet = TypingAbstractSet
-    MutableSet = TypingMutableSet
+signature = _signature
 
-    Sequence = TypingSequence
-    MutableSequence = TypingMutableSequence
-    MutableMapping = TypingMutableMapping
-    Mapping = TypingMapping
-    FrozenSetSubscriptable = FrozenSet
-    TupleSubscriptable = Tuple
+if sys.version_info >= (3, 10):
+    signature = partial(_signature, eval_str=True)
 
-    from collections import Counter as ColCounter
-    from typing import Counter, Union, _GenericAlias
-    from typing_extensions import Annotated, NotRequired, Required
-    from typing_extensions import get_origin as te_get_origin
-
-    if is_py38:
-        from typing import TypedDict, _TypedDictMeta
-    else:
-        _TypedDictMeta = None
-        TypedDict = ExtensionsTypedDict
-
-    def is_annotated(type) -> bool:
-        return te_get_origin(type) is Annotated
-
-    def is_tuple(type):
-        return type in (Tuple, tuple) or (
-            type.__class__ is _GenericAlias and issubclass(type.__origin__, Tuple)
-        )
-
-    def is_union_type(obj):
-        return (
-            obj is Union or isinstance(obj, _GenericAlias) and obj.__origin__ is Union
-        )
-
-    def get_newtype_base(typ: Any) -> Optional[type]:
-        supertype = getattr(typ, "__supertype__", None)
-        if (
-            supertype is not None
-            and getattr(typ, "__qualname__", "") == "NewType.<locals>.new_type"
-            and typ.__module__ in ("typing", "typing_extensions")
-        ):
-            return supertype
-        return None
-
-    def is_sequence(type: Any) -> bool:
-        return type in (List, list, Tuple, tuple) or (
-            type.__class__ is _GenericAlias
-            and (
-                type.__origin__ not in (Union, Tuple, tuple)
-                and issubclass(type.__origin__, TypingSequence)
-            )
-            or (type.__origin__ in (Tuple, tuple) and type.__args__[1] is ...)
-        )
-
-    def is_deque(type: Any) -> bool:
-        return (
-            type in (deque, Deque)
-            or (type.__class__ is _GenericAlias and issubclass(type.__origin__, deque))
-            or type.__origin__ is deque
-        )
-
-    def is_mutable_set(type):
-        return type is set or (
-            type.__class__ is _GenericAlias and issubclass(type.__origin__, MutableSet)
-        )
-
-    def is_frozenset(type):
-        return type is frozenset or (
-            type.__class__ is _GenericAlias and issubclass(type.__origin__, FrozenSet)
-        )
-
-    def is_mapping(type):
-        return type in (TypingMapping, dict) or (
-            type.__class__ is _GenericAlias
-            and issubclass(type.__origin__, TypingMapping)
-        )
-
-    bare_generic_args = {
-        List.__args__,
-        TypingSequence.__args__,
-        TypingMapping.__args__,
-        Dict.__args__,
-        TypingMutableSequence.__args__,
-        Tuple.__args__,
-        None,  # non-parametrized containers do not have `__args__ attribute in py3.7-8
-    }
-
-    def is_bare(type):
-        return getattr(type, "__args__", None) in bare_generic_args
-
-    def is_counter(type):
-        return (
-            type in (Counter, ColCounter)
-            or getattr(type, "__origin__", None) is ColCounter
-        )
-
-    if is_py38:
-        from typing import Literal
-
-        def is_literal(type) -> bool:
-            return type.__class__ is _GenericAlias and type.__origin__ is Literal
-
-    else:
-        # No literals in 3.7.
-        def is_literal(_) -> bool:
-            return False
-
-    def is_generic(obj):
-        return isinstance(obj, _GenericAlias)
-
-    def copy_with(type, args):
-        """Replace a generic type's arguments."""
-        return type.copy_with(args)
-
-    def is_typeddict(cls) -> bool:
-        return (
-            cls.__class__ is _TypedDictMeta
-            or (is_generic(cls) and (cls.__origin__.__class__ is _TypedDictMeta))
-            or (
-                ExtensionsTypedDictMeta is not None
-                and cls.__class__ is ExtensionsTypedDictMeta
-                or (
-                    is_generic(cls)
-                    and (cls.__origin__.__class__ is ExtensionsTypedDictMeta)
-                )
-            )
-        )
-
-    def get_notrequired_base(type) -> "Union[Any, Literal[NOTHING]]":
-        if get_origin(type) in (NotRequired, Required):
-            return get_args(type)[0]
-        return NOTHING
-
-else:
-    # 3.9+
+if sys.version_info >= (3, 9):
     from collections import Counter
-    from collections.abc import Mapping as AbcMapping
-    from collections.abc import MutableMapping as AbcMutableMapping
     from collections.abc import MutableSequence as AbcMutableSequence
     from collections.abc import MutableSet as AbcMutableSet
     from collections.abc import Sequence as AbcSequence
     from collections.abc import Set as AbcSet
     from types import GenericAlias
-    from typing import Annotated
-    from typing import Counter as TypingCounter
     from typing import (
+        Annotated,
         Generic,
         TypedDict,
         Union,
         _AnnotatedAlias,
         _GenericAlias,
         _SpecialGenericAlias,
-        _TypedDictMeta,
         _UnionGenericAlias,
     )
+    from typing import Counter as TypingCounter
 
     try:
         # Not present on 3.9.0, so we try carefully.
         from typing import _LiteralGenericAlias
 
         def is_literal(type) -> bool:
-            return type.__class__ is _LiteralGenericAlias
+            return type in LITERALS or (
+                isinstance(
+                    type, (_GenericAlias, _LiteralGenericAlias, _SpecialGenericAlias)
+                )
+                and type.__origin__ in LITERALS
+            )
 
-    except ImportError:
+    except ImportError:  # pragma: no cover
 
         def is_literal(_) -> bool:
             return False
@@ -346,11 +272,18 @@ else:
     def is_tuple(type):
         return (
             type in (Tuple, tuple)
-            or (type.__class__ is _GenericAlias and issubclass(type.__origin__, Tuple))
+            or (type.__class__ is _GenericAlias and is_subclass(type.__origin__, Tuple))
             or (getattr(type, "__origin__", None) is tuple)
         )
 
-    if is_py310_plus:
+    if sys.version_info >= (3, 12):
+        from typing import TypeAliasType
+
+        def is_type_alias(type: Any) -> bool:
+            """Is this a PEP 695 type alias?"""
+            return isinstance(type, TypeAliasType)
+
+    if sys.version_info >= (3, 10):
 
         def is_union_type(obj):
             from types import UnionType
@@ -366,7 +299,7 @@ else:
                 return typ.__supertype__
             return None
 
-        if is_py311_plus:
+        if sys.version_info >= (3, 11):
             from typing import NotRequired, Required
         else:
             from typing_extensions import NotRequired, Required
@@ -391,26 +324,20 @@ else:
                 return supertype
             return None
 
-    def is_typeddict(cls) -> bool:
-        return (
-            cls.__class__ is _TypedDictMeta
-            or (is_generic(cls) and (cls.__origin__.__class__ is _TypedDictMeta))
-            or (
-                ExtensionsTypedDictMeta is not None
-                and cls.__class__ is ExtensionsTypedDictMeta
-                or (
-                    is_generic(cls)
-                    and (cls.__origin__.__class__ is ExtensionsTypedDictMeta)
-                )
-            )
-        )
-
     def get_notrequired_base(type) -> "Union[Any, Literal[NOTHING]]":
+        if is_annotated(type):
+            # Handle `Annotated[NotRequired[int]]`
+            type = get_args(type)[0]
         if get_origin(type) in (NotRequired, Required):
             return get_args(type)[0]
         return NOTHING
 
     def is_sequence(type: Any) -> bool:
+        """A predicate function for sequences.
+
+        Matches lists, sequences, mutable sequences, deques and homogenous
+        tuples.
+        """
         origin = getattr(type, "__origin__", None)
         return (
             type
@@ -429,7 +356,7 @@ else:
                 type.__class__ is _GenericAlias
                 and (
                     (origin is not tuple)
-                    and issubclass(origin, TypingSequence)
+                    and is_subclass(origin, TypingSequence)
                     or origin is tuple
                     and type.__args__[1] is ...
                 )
@@ -441,26 +368,34 @@ else:
     def is_deque(type):
         return (
             type in (deque, Deque)
-            or (type.__class__ is _GenericAlias and issubclass(type.__origin__, deque))
+            or (type.__class__ is _GenericAlias and is_subclass(type.__origin__, deque))
             or (getattr(type, "__origin__", None) is deque)
         )
 
-    def is_mutable_set(type):
+    def is_mutable_set(type: Any) -> bool:
+        """A predicate function for (mutable) sets.
+
+        Matches built-in sets and sets from the typing module.
+        """
         return (
             type in (TypingSet, TypingMutableSet, set)
             or (
                 type.__class__ is _GenericAlias
-                and issubclass(type.__origin__, TypingMutableSet)
+                and is_subclass(type.__origin__, TypingMutableSet)
             )
             or (getattr(type, "__origin__", None) in (set, AbcMutableSet, AbcSet))
         )
 
-    def is_frozenset(type):
+    def is_frozenset(type: Any) -> bool:
+        """A predicate function for frozensets.
+
+        Matches built-in frozensets and frozensets from the typing module.
+        """
         return (
             type in (FrozenSet, frozenset)
             or (
                 type.__class__ is _GenericAlias
-                and issubclass(type.__origin__, FrozenSet)
+                and is_subclass(type.__origin__, FrozenSet)
             )
             or (getattr(type, "__origin__", None) is frozenset)
         )
@@ -470,18 +405,17 @@ else:
             not hasattr(type, "__origin__") and not hasattr(type, "__args__")
         )
 
-    def is_mapping(type):
+    def is_mapping(type: Any) -> bool:
+        """A predicate function for mappings."""
         return (
             type in (dict, Dict, TypingMapping, TypingMutableMapping, AbcMutableMapping)
             or (
                 type.__class__ is _GenericAlias
-                and issubclass(type.__origin__, TypingMapping)
+                and is_subclass(type.__origin__, TypingMapping)
             )
-            or (
-                getattr(type, "__origin__", None)
-                in (dict, AbcMutableMapping, AbcMapping)
+            or is_subclass(
+                getattr(type, "__origin__", type), (dict, AbcMutableMapping, AbcMapping)
             )
-            or issubclass(type, dict)
         )
 
     def is_counter(type):
@@ -490,20 +424,156 @@ else:
             or getattr(type, "__origin__", None) is Counter
         )
 
-    def is_generic(obj) -> bool:
-        return (
-            isinstance(obj, _GenericAlias)
-            or isinstance(obj, GenericAlias)
-            or is_subclass(obj, Generic)
+    def is_generic(type) -> bool:
+        """Whether `type` is a generic type."""
+        # Inheriting from protocol will inject `Generic` into the MRO
+        # without `__orig_bases__`.
+        return isinstance(type, (_GenericAlias, GenericAlias)) or (
+            is_subclass(type, Generic) and hasattr(type, "__orig_bases__")
         )
 
     def copy_with(type, args):
         """Replace a generic type's arguments."""
         if is_annotated(type):
             # typing.Annotated requires a special case.
-            return Annotated[args]  # type: ignore
+            return Annotated[args]
+        if isinstance(args, tuple) and len(args) == 1:
+            # Some annotations can't handle 1-tuples.
+            args = args[0]
         return type.__origin__[args]
 
+    def get_full_type_hints(obj, globalns=None, localns=None):
+        return get_type_hints(obj, globalns, localns, include_extras=True)
 
-def is_generic_attrs(type):
+else:
+    # 3.8
+    Set = TypingSet
+    AbstractSet = TypingAbstractSet
+    MutableSet = TypingMutableSet
+
+    Sequence = TypingSequence
+    MutableSequence = TypingMutableSequence
+    MutableMapping = TypingMutableMapping
+    Mapping = TypingMapping
+    FrozenSetSubscriptable = FrozenSet
+    TupleSubscriptable = Tuple
+
+    from collections import Counter as ColCounter
+    from typing import Counter, Generic, TypedDict, Union, _GenericAlias
+
+    from typing_extensions import Annotated, NotRequired, Required
+    from typing_extensions import get_origin as te_get_origin
+
+    def is_annotated(type) -> bool:
+        return te_get_origin(type) is Annotated
+
+    def is_tuple(type):
+        return type in (Tuple, tuple) or (
+            type.__class__ is _GenericAlias and is_subclass(type.__origin__, Tuple)
+        )
+
+    def is_union_type(obj):
+        return (
+            obj is Union or isinstance(obj, _GenericAlias) and obj.__origin__ is Union
+        )
+
+    def get_newtype_base(typ: Any) -> Optional[type]:
+        supertype = getattr(typ, "__supertype__", None)
+        if (
+            supertype is not None
+            and getattr(typ, "__qualname__", "") == "NewType.<locals>.new_type"
+            and typ.__module__ in ("typing", "typing_extensions")
+        ):
+            return supertype
+        return None
+
+    def is_sequence(type: Any) -> bool:
+        return type in (List, list, Tuple, tuple) or (
+            type.__class__ is _GenericAlias
+            and (
+                type.__origin__ not in (Union, Tuple, tuple)
+                and is_subclass(type.__origin__, TypingSequence)
+            )
+            or (type.__origin__ in (Tuple, tuple) and type.__args__[1] is ...)
+        )
+
+    def is_deque(type: Any) -> bool:
+        return (
+            type in (deque, Deque)
+            or (type.__class__ is _GenericAlias and is_subclass(type.__origin__, deque))
+            or type.__origin__ is deque
+        )
+
+    def is_mutable_set(type) -> bool:
+        return type in (set, TypingAbstractSet) or (
+            type.__class__ is _GenericAlias
+            and is_subclass(type.__origin__, (MutableSet, TypingAbstractSet))
+        )
+
+    def is_frozenset(type):
+        return type is frozenset or (
+            type.__class__ is _GenericAlias and is_subclass(type.__origin__, FrozenSet)
+        )
+
+    def is_mapping(type: Any) -> bool:
+        """A predicate function for mappings."""
+        return (
+            type in (TypingMapping, dict)
+            or (
+                type.__class__ is _GenericAlias
+                and is_subclass(type.__origin__, TypingMapping)
+            )
+            or is_subclass(
+                getattr(type, "__origin__", type), (dict, AbcMutableMapping, AbcMapping)
+            )
+        )
+
+    bare_generic_args = {
+        List.__args__,
+        TypingSequence.__args__,
+        TypingMapping.__args__,
+        Dict.__args__,
+        TypingMutableSequence.__args__,
+        Tuple.__args__,
+        None,  # non-parametrized containers do not have `__args__ attribute in py3.7-8
+    }
+
+    def is_bare(type):
+        return getattr(type, "__args__", None) in bare_generic_args
+
+    def is_counter(type):
+        return (
+            type in (Counter, ColCounter)
+            or getattr(type, "__origin__", None) is ColCounter
+        )
+
+    def is_literal(type) -> bool:
+        return type in LITERALS or (
+            isinstance(type, _GenericAlias) and type.__origin__ in LITERALS
+        )
+
+    def is_generic(obj):
+        return isinstance(obj, _GenericAlias) or (
+            is_subclass(obj, Generic) and hasattr(obj, "__orig_bases__")
+        )
+
+    def copy_with(type, args):
+        """Replace a generic type's arguments."""
+        return type.copy_with(args)
+
+    def get_notrequired_base(type) -> "Union[Any, Literal[NOTHING]]":
+        if is_annotated(type):
+            # Handle `Annotated[NotRequired[int]]`
+            type = get_origin(type)
+
+        if get_origin(type) in (NotRequired, Required):
+            return get_args(type)[0]
+        return NOTHING
+
+    def get_full_type_hints(obj, globalns=None, localns=None):
+        return get_type_hints(obj, globalns, localns)
+
+
+def is_generic_attrs(type) -> bool:
+    """Return True for both specialized (A[int]) and unspecialized (A) generics."""
     return is_generic(type) and has(type.__origin__)
